package de.dhbwloe.campusapp;

import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.widget.TextView;

import java.lang.reflect.Method;

import de.dhbwloe.campusapp.database.DatabaseManager;
import de.dhbwloe.campusapp.fragments.AppSearch;
import de.dhbwloe.campusapp.fragments.Dashboard;
import de.dhbwloe.campusapp.fragments.FirstRun;
import de.dhbwloe.campusapp.fragments.Impressum;
import de.dhbwloe.campusapp.fragments.Mensa;
import de.dhbwloe.campusapp.fragments.MensaCard;
import de.dhbwloe.campusapp.fragments.News;
import de.dhbwloe.campusapp.fragments.PopupFragment;
import de.dhbwloe.campusapp.fragments.Settings;
import de.dhbwloe.campusapp.fragments.SplashScreen;
import de.dhbwloe.campusapp.fragments.Vorlesungsplan;
import de.dhbwloe.campusapp.fragments.WebBrowser;
import de.dhbwloe.campusapp.fragments.WifiSettings;
import de.dhbwloe.campusapp.nfcreader.NfcCardInterface;
import de.dhbwloe.campusapp.nfcreader.NfcCardListener;
import de.dhbwloe.campusapp.database.NfcCardData;
import de.dhbwloe.campusapp.search.SearchIndices;

/** CampusAppContext
 * A context, that is pushed to each fragment.
 * This context will be persistent during app execution.
 */
public class CampusAppContext {
    private class AppPage {
        String name;
        Class<?> fragment;
        int fragementType;

        public AppPage(String name, Class<?> fragment) {
            this.name = name;
            this.fragment = fragment;
        }

        public AppPage(String name, Class<?> fragment, int type) {
            this.name = name;
            this.fragment = fragment;
            this.fragementType = type;
        }
    }
    private final AppPage[] PAGES = {
            new AppPage("SplashScreen", SplashScreen.class),
            new AppPage("Dashboard", Dashboard.class),
            new AppPage("AppSearch", AppSearch.class),
            new AppPage("Vorlesungsplan", Vorlesungsplan.class),
            new AppPage("Mensa", Mensa.class),
            new AppPage("MensaCard", MensaCard.class, 3),
            new AppPage("News", News.class),
            new AppPage("WifiSettings", WifiSettings.class),
            new AppPage("FirstRun", FirstRun.class),
            new AppPage("Settings", Settings.class),
            new AppPage("Impressum", Impressum.class),
            new AppPage("WebBrowser", WebBrowser.class),
            new AppPage("WebBrowserPopup", WebBrowser.class, 3)
    };

    private static CampusAppContext instance;
    public static CampusAppContext getInstance() {
        Log.i("AppContext", "Request new context instance");
        return instance;
    }

    private Activity oMainActivity;
    private NavigationManager oNavigationManager;
    private DatabaseManager oDatabaseManager;
    private NfcCardListener oNfcCardListener;
    private Bundle oContextVariables;

    public CampusAppContext(CampusApp mainActivity, int fragmentContainerId) {
        final CampusAppContext AppContext = this;
        instance = this;
        oMainActivity = mainActivity;
        oContextVariables = new Bundle();
        oNavigationManager = new NavigationManager(this, fragmentContainerId);
        oDatabaseManager = new DatabaseManager(this);
        oNfcCardListener = new NfcCardListener(this);

        for(int i = 0; i < PAGES.length; i++)
            oNavigationManager.registerPage(PAGES[i].name, PAGES[i].fragment, PAGES[i].fragementType);

        oNfcCardListener.registerNfcCardInterface(new NfcCardInterface() {
            @Override
            public void onNfcReaderStateChanged(boolean state) {

            }

            @Override
            public void onNfcReaderReceived(NfcCardData data) {
                if (data != null)
                    AppContext.onNfcCardDataReceived(data);
            }
        });
    }

    public Activity getMainActivity() {
        return oMainActivity;
    }
    public void setMainActivity(Activity activity) {
        oMainActivity = activity;
    }

    public NavigationManager getNavigationManager() {
        return oNavigationManager;
    }

    public void setTitle(String title) {
        PopupFragment popup = oNavigationManager.getDialog();
        if(popup != null) {
            if(popup.getDialog() != null)
                popup.getDialog().setTitle(title);
        } else {
            TextView titleView = (TextView)oMainActivity.findViewById(R.id.title);
            titleView.setText(title);
        }
    }

    public DatabaseManager getDatabaseManager() {
        return oDatabaseManager;
    }

    public void addDefaultSearchIndexes() {
        for(int i = 0; i < PAGES.length; i++) {
            try {
                Method m = PAGES[i].fragment.getMethod("GetSearchIndices");
                Object result = m.invoke(null);
                SearchIndices[] indices = (SearchIndices[]) result;
                addSearchIndices(indices);
            } catch (Exception e) {
            }
        }
    }

    public void addSearchIndices(SearchIndices[] indices) {
        oDatabaseManager.addSearchIndices(indices);
    }

    public NfcCardListener getNfcCardListener() {
        return oNfcCardListener;
    }

    private void onNfcCardDataReceived(NfcCardData data) {
        Bundle bundle = new Bundle();
        double cardBalance = data.getBalance() / 100.0;
        bundle.putDouble("balance", cardBalance);
        bundle.putDouble("transaction", data.getLastTransaction() / 100.0);

        oContextVariables.putInt("nfcCardUniqueId", data.getUniqueId());
        oContextVariables.putDouble("nfcCardBalance", cardBalance);

        String pagename = oNavigationManager.getCurrentPageName();
        if(pagename != null && pagename.equalsIgnoreCase("MensaCard")) {
            MensaCard fragment = (MensaCard) oNavigationManager.getCurrentFragment();
            fragment.showNfcCardData(bundle);
        } else
            oNavigationManager.navigatePage("MensaCard", bundle);
    }

    public Bundle getContextVariables() {
        return oContextVariables;
    }

    public String getResString(int id, Object... arguments) {
        String format = getResString(id);
        String res = String.format(format, arguments);
        return res;
    }
    public String getResString(int id) {
        String str = oMainActivity.getResources().getString(id);
        return str;
    }

}
