package de.dhbwloe.campusapp.fragments;


import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.ProgressBar;

import java.util.ArrayList;
import java.util.List;

import de.dhbwloe.campusapp.CampusAppContext;
import de.dhbwloe.campusapp.CampusAppFragment;
import de.dhbwloe.campusapp.R;
import de.dhbwloe.campusapp.search.DhbwSearchHelper;
import de.dhbwloe.campusapp.search.SearchIndices;
import de.dhbwloe.campusapp.search.SearchResultListener;
import de.dhbwloe.campusapp.search.SearchTarget;

/**
 * A simple {@link Fragment} subclass.
 */
public class AppSearchDhbw extends CampusAppFragment implements AppSearchProvider {
    private String sSearchQuery;
    private boolean bQueryExecuted = false;
    private AppSearchListAdapter appSearchAdapter;
    private DhbwSearchHelper seachHelper;
    private ArrayList<AppSearchListItem> searchResultItems = new ArrayList<AppSearchListItem>();
    private ProgressBar progressBar;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        Log.i("AppSearchDhbw", "Event: onCreate");
        super.onCreate(savedInstanceState);
    }

    @Override
    public void onSaveInstanceState(Bundle savedInstanceState) {
        Log.i("AppSearchDhbw", "Event: onSaveInstanceState");
        Bundle bundle = new Bundle();
        bundle.putString("query", sSearchQuery);
        bundle.putBoolean("executed", bQueryExecuted);
        savedInstanceState.putBundle("SearchDhbw", bundle);
        super.onSaveInstanceState(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        Log.i("AppSearchDhbw", "Event: onCreateView");
        Bundle savedState;
        if((savedState = getArguments()) != null)
            sSearchQuery = savedState.getString("query");
        else if(savedInstanceState != null && (savedState = savedInstanceState.getBundle("SearchDhbw")) != null) {
            Log.i("AppSearchDhbw", "Parsed instance state");
            sSearchQuery = savedState.getString("query");
            if(savedState.getBoolean("executed")) {
                bQueryExecuted = true;
            }
        }
        if(AppContext == null)
            AppContext = CampusAppContext.getInstance();
        oFragmentView = inflater.inflate(R.layout.fragment_appsearch_list, container, false);

        Bundle args = getArguments();
        if(args != null) {
            String queryString = args.getString("query");
            if(queryString != null)
                sSearchQuery = queryString;
        }

        ListView searchResultList = (ListView) oFragmentView.findViewById(R.id.searchResultItems);
        appSearchAdapter = new AppSearchListAdapter(oFragmentView.getContext(), R.layout.fragment_appsearch_listitem, searchResultItems);
        searchResultList.setAdapter(appSearchAdapter);

        searchResultList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                AppSearchListItem item = searchResultItems.get(position);
                if (item == null)
                    return;
                navigateToResult(item);
            }
        });

        return oFragmentView;
    }

    public void executeSearch(boolean reset) {
        Log.i("AppSearchDhbw", "Action: executeSearch");
        if(AppContext == null)
            AppContext = CampusAppContext.getInstance();
        if(bQueryExecuted && !reset)
            return;
        bQueryExecuted = true;
        progressBar = (ProgressBar) oFragmentView.findViewById(R.id.searchProgress);
        progressBar.setVisibility(View.VISIBLE);

        searchResultItems.clear();

        if(seachHelper == null)
            seachHelper = new DhbwSearchHelper();
        seachHelper.search(sSearchQuery, new SearchResultListener() {
            @Override
            public void onSearchResultsReceived(List<AppSearchListItem> results) {
                Log.i("SearchDhbw", "Success: "+results.size());
                for(AppSearchListItem result : results)
                    searchResultItems.add(result);
                progressBar.setVisibility(View.GONE);
                if(appSearchAdapter != null)
                    appSearchAdapter.notifyDataSetChanged();
                else
                    Log.i("SearchDhbw", "appSearchAdapter is null in executeSearch");
            }

            @Override
            public void onSearchFailed(String error) {
                Log.i("SearchDhbw", "Failed: " + error);
                progressBar.setVisibility(View.GONE);
            }
        });
    }

    private void navigateToResult(AppSearchListItem result) {
        SearchTarget target = result.getTarget();

        target.navigate(AppContext.getNavigationManager());
    }

}
