package de.dhbwloe.campusapp.nfcreader;

/**
 * Created by stefan on 20.01.16.
 */
public class Hex {
    /**
     * Converts a hex string (Format: "XX XX XX ..") into a Byte array
     * @param hexString
     * @return Byte array
     */
    public static Byte[] stringToHex(String hexString) {
        if (hexString == null)
            return null;
        hexString = hexString.replace(" ", "");
        final int bytesCount = hexString.length() / 2;
        if (2 * bytesCount != hexString.length()) {
            throw new IllegalArgumentException(
                    "Hex string must have an even number of digits");
        }

        Byte[] result = new Byte[bytesCount];
        for (int i = 0; i < hexString.length(); i += 2) {
            result[i / 2] = (byte) Integer.parseInt(hexString.substring(i, i + 2), 16);
        }
        return result;
    }

    /**
     *  Converts a Byte array into a hex string (Format "xx xx xx ..")
     * @param bytes
     * @return hex string
     */
    public static String hexToString(Byte[] bytes) {
        StringBuffer buff = new StringBuffer();
        for (byte b : bytes) {
            buff.append(String.format("%02X", b));
        }
        return buff.toString();
    }

    /**
     * Extracts parts of the Byte array into a seperate array
     * @param start Start position of sub array
     * @param stop Stop position of sub array
     * @param array Array to extract from
     * @return sub array
     */
    public static Byte[] subByteArray(int start, int stop, Byte[] array) {
        Byte[] returnArray = new Byte[stop-start];
        int position = 0;
        for (int i = start; i<stop; i++) {
            returnArray[position] = array[i];
            position++;
        }
        return returnArray;
    }


    /**
     * Reverses byte order in byte array (to convert from LSB and MSB and backwards)
     * @param bytes byte array
     * @return reversed byte array
     */
    public static Byte[] reverseByteOrder(Byte[] bytes) {
        Byte[] returnBytes = new Byte[bytes.length];
        for (int i=0; i<bytes.length; i++) {
            returnBytes[bytes.length - 1 - i] = bytes[i];
        }
        return returnBytes;
    }

    /**
     * Converts byte array to integer
     * @param bytes
     * @return interger
     */
    public static int hexToInteger(Byte[] bytes) {
        String hexString = hexToString(bytes);
        return Integer.parseInt(hexString, 16);
    }

    /**
     * Converts a Byte[] array to byte[] array (Object to primitive)
     * @param oBytes
     * @return
     */
    public static byte[] bytesToPrimitives(Byte[] oBytes)
    {
        byte[] bytes = new byte[oBytes.length];

        for(int i = 0; i < oBytes.length; i++) {
            bytes[i] = oBytes[i];
        }

        return bytes;
    }

    /**
     * Converts a byte[] array to Byte[] array (Primitive to object)
     * @param bytesPrim
     * @return
     */
    public static Byte[] bytesToObjects(byte[] bytesPrim) {
        Byte[] bytes = new Byte[bytesPrim.length];

        int i = 0;
        for (byte b : bytesPrim) bytes[i++] = b; // Autoboxing

        return bytes;
    }

    public static int byteArrayToInt(byte[] b) {
        return byteArrayToInt(b, 0);
    }

    public static int byteArrayToInt(byte[] b, int offset) {
        return byteArrayToInt(b, offset, b.length);
    }

    public static int byteArrayToInt(byte[] b, int offset, int length) {
        return (int) byteArrayToLong(b, offset, length);
    }

    public static long byteArrayToLong(byte[] b, int offset, int length) {
        if (b.length < length)
            throw new IllegalArgumentException("length must be less than or equal to b.length");

        long value = 0;
        for (int i = 0; i < length; i++) {
            int shift = (length - 1 - i) * 8;
            value += (b[i + offset] & 0x000000FF) << shift;
        }
        return value;
    }
}
