package de.dhbwloe.campusapp.search;

import android.util.Log;

import com.loopj.android.http.AsyncHttpResponseHandler;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;

import java.io.UnsupportedEncodingException;
import java.util.List;

import cz.msebera.android.httpclient.Header;
import de.dhbwloe.campusapp.fragments.AppSearchListItem;

/**
 * Created by pk910 on 25.01.2016.
 */
public abstract class SearchHelper {

    protected boolean bRequestRunning = false;
    protected boolean bHtmlParser;
    protected SearchResultListener oRequestListener = null;

    public SearchHelper(boolean htmlparser) {
        bHtmlParser = htmlparser;
    }

    protected void performWebRequest(String keywords) {
        AsyncHttpResponseHandler handler = new AsyncHttpResponseHandler() {
            @Override
            public void onStart() {
            }
            @Override
            public void onSuccess(int statusCode, Header[] headers, byte[] response) {
                boolean parsedHttp = false;
                if(bHtmlParser) {
                    try {
                        Document document = Jsoup.parse(new String(response));

                        parsedHttp = true;
                        bRequestRunning = false;
                        try {
                            List<AppSearchListItem> results = onHtmlDocumentReceived(document);
                            oRequestListener.onSearchResultsReceived(results);
                        } catch(Exception e) {
                            oRequestListener.onSearchFailed(e.getMessage());
                        }
                    } catch(Exception e) {
                    }
                }
                if(!parsedHttp) {
                    try {
                        List<AppSearchListItem> results = onPlainTextReceived(response);
                        oRequestListener.onSearchResultsReceived(results);
                    } catch(Exception e) {
                        oRequestListener.onSearchFailed(e.getMessage());
                    }
                }
                bRequestRunning = false;
            }
            @Override
            public void onFailure(int statusCode, Header[] headers, byte[] errorResponse, Throwable e) {
                String error = null;
                try {
                    error = new String(errorResponse, "US-ASCII");
                } catch (Exception e1) {
                }
                Log.i("HTTPClient", "  Error: " + statusCode + " - " + error);
                bRequestRunning = false;
                oRequestListener.onSearchFailed(statusCode + ": " + error);
            }
            @Override
            public void onRetry(int retryNo) {
            }
        };
        processWebRequest(handler, keywords);
    }

    public void search(String keywords, SearchResultListener resultlistener) {
        if(bRequestRunning) {
            resultlistener.onSearchFailed("another request is already running!");
            return;
        }
        oRequestListener = resultlistener;
        bRequestRunning = true;

        performWebRequest(keywords);
    }

    protected abstract void processWebRequest(AsyncHttpResponseHandler handler, String keywords);
    protected abstract List<AppSearchListItem> onHtmlDocumentReceived(Document document);
    protected abstract List<AppSearchListItem> onPlainTextReceived(byte[] data);
}
